@@ -1054,8 +1054,7 @@ def transpose(a, perm=None, conjugate=False):
         A transposed Tensor.
     """
     # TODO conjugate
-    trans_obj = P.Transpose()
-    outputs = trans_obj(a, perm)
+    outputs = msnp.transpose(a, perm)
     print(outputs)
 
 
@@ -1171,6 +1170,8 @@ def floor(x):
 
 def gather(params, indices, axis=None):
     op = P.Gather()
+    if axis is None:
+        axis = 0
     return op(params, indices, axis)
 
 
@@ -1591,10 +1592,7 @@ def reduce_std(x, axis=None, keepdims=False):
 
 
 def reduce_sum(x, axis=None, keepdims=False):
-    op = P.ReduceSum(keep_dims=keepdims)
-    if axis is None:
-        return op(x)
-    return op(x, axis=axis)
+    return msnp.sum(x, axis=axis, keepdims=keepdims)
 
 
 def reduce_variance(x, axis=None, keepdims=False):
@@ -1730,11 +1728,15 @@ def tanh(x):
 
 def any(x, axis=None, keepdims=False):
     op = P.ReduceAny(keep_dims=keepdims)
+    if axis is None:
+        return op(x)
     return op(x, axis)
 
 
 def all(x, axis=None, keepdims=False):
     op = P.ReduceAll(keep_dims=keepdims)
+    if axis is None:
+        return op(x)
     return op(x, axis)
 
 
@@ -1780,8 +1782,7 @@ def zeros_like(x, dtype=None):
 
 
 def squeeze(x, axis=None):
-    op = P.Squeeze(axis)
-    return op(x)
+    return msnp.squeeze(x, axis)
 
 
 def unsorted_segment_sum(x, segment_ids, num_segments):
@@ -1793,7 +1794,7 @@ def unsorted_segment_sum(x, segment_ids, num_segments):
 def unsorted_segment_mean(x, segment_ids, num_segments):
     segment_ids = ms.Tensor(segment_ids)
     op = P.UnsortedSegmentSum()
-    x_one =  msnp.ones_like(x, dtype=x.dtype)
+    x_one = msnp.ones_like(x, dtype=x.dtype)
     sum = op(x, segment_ids, num_segments)
     one = op(x_one, segment_ids, num_segments)
     return sum/one
@@ -1816,4 +1817,16 @@ def set_seed(seed):
 
 def is_tensor(x):
 
-    return isinstance(x, ms.Tensor)
\ No newline at end of file
+    return isinstance(x, ms.Tensor)
+
+def tensor_scatter_nd_update(tensor, indices, updates):
+    if not isinstance(tensor,  ms.Tensor) or not isinstance(updates, ms.Tensor):
+        raise TypeError("tensor, updates should be Tensor, but got tensor type is {}, "
+                        "and updates type is {}.".format(type(tensor), type(updates)))
+    indices = ms.Tensor(indices)
+    op = ms.ops.TensorScatterUpdate()
+    return op(tensor, indices, updates)
+
+def diag(input, diagonal=0):
+
+    return ms.numpy.diag(input, diagonal)
\ No newline at end of file
