@@ -2,6 +2,8 @@
 # -*- coding: utf-8 -*-
 
 from __future__ import absolute_import, division, print_function
+
+import paddle
 import paddle as pd
 import paddle.nn as nn
 import numpy as np
@@ -9,26 +11,61 @@ import paddle.nn.functional as F
 from .paddle_nn import nchw_to_nhwc, nhwc_to_nchw, preprocess_2d_format, preprocess_1d_format, preprocess_3d_format
 import random
 
-_dtypeDict = [
-    "float16", "float32", "float64", "int8", "int16", "int32", "int64", "uint8", "uint16", "uint32", "uint64", "bool",
-    "complex64", "complex128"
-]
-# TODO NotImplemented
-DType = None
-float16 = "float16"
-float32 = "float32"
-float64 = "float64"
-int8 = "int8"
-int16 = "int16"
-int32 = "int32"
-int64 = "int64"
-uint8 = "uint8"
-uint16 = "uint16"
-uint32 = "uint32"
-uint64 = "uint64"
-bool = "bool"
-complex64 = "complex64"
-complex128 = "complex128"
+if paddle.__version__ < '2.2.2':
+    _dtypeDict = [
+        "float16", "float32", "float64", "int8", "int16", "int32", "int64", "uint8", "uint16", "uint32", "uint64", "bool",
+        "complex64", "complex128"
+    ]
+    # TODO NotImplemented
+    DType = None
+    float16 = "float16"
+    float32 = "float32"
+    float64 = "float64"
+    int8 = "int8"
+    int16 = "int16"
+    int32 = "int32"
+    int64 = "int64"
+    uint8 = "uint8"
+    uint16 = "uint16"
+    uint32 = "uint32"
+    uint64 = "uint64"
+    bool = "bool"
+    complex64 = "complex64"
+    complex128 = "complex128"
+else:
+    _dtypeDict = {
+        'DType': paddle.dtype ,
+        'float16': paddle.float16,
+        'float32': paddle.float32,
+        'float64': paddle.float64,
+        'int8': paddle.int8,
+        'int16': paddle.int16,
+        'int32': paddle.int32,
+        'int64': paddle.int64,
+        'uint8': paddle.uint8,
+        'uint16': None,
+        'uint32': None,
+        'uint64': None,
+        'bool': paddle.bool,
+        'complex64': paddle.complex64,
+        'complex128': paddle.complex128
+    }
+    # TODO NotImplemented
+    DType = paddle.dtype
+    float16 = paddle.float16
+    float32 = paddle.float32
+    float64 = paddle.float64
+    int8 = paddle.int8
+    int16 = paddle.int16
+    int32 = paddle.int32
+    int64 = paddle.int64
+    uint8 = paddle.uint8
+    uint16 = None
+    uint32 = None
+    uint64 = None
+    bool = paddle.bool
+    complex64 = paddle.complex64
+    complex128 = paddle.complex128
 
 
 def _getter(init_fn, **kwargs):
@@ -824,7 +861,15 @@ def transpose(a, perm=None, conjugate=False):
     -------
         A transposed Tensor.
     """
-
+    if perm == None:
+        if len(a.shape) <= 2:
+            return pd.t(a)
+        if len(a.shape) == 3:
+            perm = [2, 1, 0]
+        if len(a.shape) == 4:
+            perm = [3, 2, 1, 0]
+        if len(a.shape) == 5:
+            perm = [4, 3, 2, 1, 0]
     return pd.transpose(a, perm)
 
 
@@ -1503,4 +1548,19 @@ def set_seed(seed):
 
 def is_tensor(x):
 
-    return pd.is_tensor(x)
\ No newline at end of file
+    return pd.is_tensor(x)
+
+
+def tensor_scatter_nd_update(tensor, indices, updates):
+    tensor = paddle.to_tensor(tensor)
+    indices = paddle.to_tensor(indices)
+    updates = paddle.to_tensor(updates)
+    a = pd.scatter_nd(indices, pd.ones_like(updates), tensor.shape)
+    a = pd.multiply(tensor, -a)
+    tensor = tensor + a
+    x = pd.scatter_nd_add(tensor, indices, updates)
+    return x
+
+def diag(input, diagonal=0):
+
+    return paddle.diag(input, diagonal)
\ No newline at end of file
