@@ -1012,7 +1012,9 @@ def floor(x):
     return torch.floor(x)
 
 
-def gather(params, indices, axis = 0):
+def gather(params, indices, axis = None):
+    if axis is None:
+        axis = 0
     if axis < 0:
         axis = len(params.shape) + axis
     if axis == 0:
@@ -1522,11 +1524,16 @@ def tanh(x):
 
 
 def any(x, axis=None, keepdims=False):
-    return torch.any(x, dim=axis, keepdim=keepdims)
-
+    if axis is not None:
+        return torch.any(x, dim=axis, keepdim=keepdims)
+    else:
+        return torch.any(x)
 
 def all(x, axis=None, keepdims=False):
-    return  torch.all(x, dim=axis, keepdim=keepdims)
+    if axis is not None:
+        return torch.all(x, dim=axis, keepdim=keepdims)
+    else:
+        return torch.all(x)
 
 
 def logical_and(x, y):
@@ -1630,4 +1637,16 @@ def set_seed(seed):
 
 def is_tensor(x):
 
-    return isinstance(x, torch.Tensor)
\ No newline at end of file
+    return isinstance(x, torch.Tensor)
+
+def tensor_scatter_nd_update(tensor, indices, updates):
+    tensor = torch.tensor(tensor)
+    indices = torch.tensor(indices)
+    updates = torch.tensor(updates)
+    indices = torch.flatten(indices)
+    tensor[indices] = updates
+    return tensor
+
+def diag(input, diagonal=0):
+
+    return torch.diag(input, diagonal)
