@@ -27,6 +27,8 @@ def padding_format(padding):
         padding = "valid"
     elif padding == None:
         padding = None
+    elif isinstance(padding, tuple) or isinstance(padding, int):
+        return padding
     else:
         raise Exception("Unsupported padding: " + str(padding))
     return padding
@@ -573,9 +575,13 @@ def same_padding(input, weight, strides, dilations):
 class Conv2D(object):
 
     def __init__(self, strides, padding, data_format='NHWC', dilations=None, out_channel=None, k_size=None, groups=1):
-        self.strides = (strides[1], strides[2])
-        self.dilations = (dilations[1], dilations[2])
         self.data_format, self.padding = preprocess_2d_format(data_format, padding)
+        if self.data_format is 'NHWC':
+            self.strides = (strides[1], strides[2])
+            self.dilations = (dilations[1], dilations[2])
+        elif self.data_format is 'NCHW':
+            self.strides = (strides[2], strides[3])
+            self.dilations = (dilations[2], dilations[3])
         self.groups = groups
 
     def __call__(self, input, filters):
@@ -644,13 +650,14 @@ def conv2d(input, filters, strides, padding, data_format='NHWC', dilations=None)
 class Conv3D(object):
 
     def __init__(self, strides, padding, data_format='NDHWC', dilations=None, out_channel=None, k_size=None):
-        if data_format is 'NDHWC':
+        self.data_format, self.padding = preprocess_3d_format(data_format, padding)
+        if self.data_format is 'NDHWC':
             self._strides = (strides[1], strides[2], strides[3])
             self._dilations = (dilations[1], dilations[2], dilations[3])
-        elif data_format is 'NCDHW':
+        elif self.data_format is 'NCDHW':
             self._strides = (strides[2], strides[3], strides[4])
             self._dilations = (dilations[2], dilations[3], dilations[4])
-        self.data_format, self.padding = preprocess_3d_format(data_format, padding)
+
 
     def __call__(self, input, filters):
         if self.data_format == 'NDHWC':
@@ -793,17 +800,17 @@ class MaxPool(object):
             if self.padding in ['SAME', 'same']:
                 out = self.maxpool1d_same_padding(inputs)
             else:
-                out = F.max_pool1d(inputs, self.ksize, self.strides)
+                out = F.max_pool1d(inputs, self.ksize, self.strides, padding=self.padding)
         if len(inputs.shape) == 4:
             if self.padding in ['SAME', 'same']:
                 out = self.maxpool2d_same_padding(inputs)
             else:
-                out = F.max_pool2d(inputs, self.ksize, (self.strides[1], self.strides[2]))
+                out = F.max_pool2d(inputs, self.ksize, (self.strides[1], self.strides[2]), padding=self.padding)
         if len(inputs.shape) == 5:
             if self.padding in ['SAME', 'same']:
                 out = self.maxpool3d_same_padding(inputs)
             else:
-                out = F.max_pool3d(inputs, self.ksize, (self.strides[1], self.strides[2], self.strides[3]))
+                out = F.max_pool3d(inputs, self.ksize, (self.strides[1], self.strides[2], self.strides[3]), padding=self.padding)
 
         if self.data_format == 'channels_last':
             return nchw_to_nhwc(out)
@@ -895,17 +902,17 @@ class AvgPool(object):
             if self.padding in ['SAME', 'same']:
                 out = self.avgpool1d_same_padding(inputs)
             else:
-                out = F.avg_pool1d(inputs, self.ksize, self.strides)
+                out = F.avg_pool1d(inputs, self.ksize, self.strides, padding=self.padding)
         if len(inputs.shape) == 4:
             if self.padding in ['SAME', 'same']:
                 out = self.avgpool2d_same_padding(inputs)
             else:
-                out = F.avg_pool2d(inputs, self.ksize, (self.strides[1], self.strides[2]))
+                out = F.avg_pool2d(inputs, self.ksize, (self.strides[1], self.strides[2]), padding=self.padding)
         if len(inputs.shape) == 5:
             if self.padding in ['SAME', 'same']:
                 out = self.avgpool3d_same_padding(inputs)
             else:
-                out = F.avg_pool3d(inputs, self.ksize, (self.strides[1], self.strides[2], self.strides[3]))
+                out = F.avg_pool3d(inputs, self.ksize, (self.strides[1], self.strides[2], self.strides[3]), padding=self.padding)
 
         if self.data_format == 'channels_last':
             return nchw_to_nhwc(out)
