@@ -6,6 +6,7 @@ import tensorlayerx as tlx
 from tensorlayerx.files import utils
 from tensorlayerx import logging
 import numpy as np
+from queue import Queue
 from tensorlayerx.nn.initializers import *
 
 if tlx.BACKEND == 'mindspore':
@@ -38,6 +39,22 @@ _initializers_dict = {
 }
 
 
+def check_parameter(parameter, dim='2d'):
+    if dim == '2d':
+        if isinstance(parameter, int):
+            out = (parameter, parameter)
+        else:
+            out = parameter
+    elif dim == '3d':
+        if isinstance(parameter, int):
+            out = (parameter, parameter, parameter)
+        else:
+            out = parameter
+    else:
+        raise ("dim must be 2d or 3d.")
+    return out
+
+
 def str2init(initializer):
     if isinstance(initializer, str):
         if initializer not in _initializers_dict.keys():
@@ -437,3 +454,134 @@ def weight_reshape(weight, reshape=False):
         if len(weight.shape) == 5:
             weight = np.moveaxis(weight, (3, 4), (1, 0))
     return weight
+
+def tolist(tensors):
+    if isinstance(tensors, list) or isinstance(tensors, tuple):
+        ntensors = list()
+        for t in tensors:
+            ntensors += tolist(t)
+        return ntensors
+    else:
+        return [tensors]
+
+def construct_graph(inputs, outputs):
+    """construct computation graph for model using ModuleNode object"""
+    all_layers = []
+    node_by_depth = []
+
+    input_tensors_list = inputs if isinstance(inputs, list) else inputs
+
+    queue_node = Queue()
+    # BFS to visit all nodes that should be involved in the computation graph
+    output_tensors_list = outputs if isinstance(outputs, list) else [outputs]
+    output_nodes = [tensor._info[0] for tensor in output_tensors_list]
+
+    visited_node_names = set()
+    for out_node in output_nodes:
+        if out_node.visited:
+            continue
+        queue_node.put(out_node)
+
+        while not queue_node.empty():
+            cur_node = queue_node.get()
+            in_nodes = cur_node.in_nodes
+
+            for node in in_nodes:
+                node.out_nodes.append(cur_node)
+                if not node.visited:
+                    queue_node.put(node)
+                    node.visited = True
+                    if node.node_name not in visited_node_names:
+                        visited_node_names.add(node.node_name)
+                    # else have multiple layers with the same name
+                    else:
+                        raise ValueError(
+                            'Layer name \'%s\' has already been used by another layer. Please change the layer name.'
+                            % node.layer.name
+                        )
+
+    # construct the computation graph in top-sort order
+    cur_depth = [tensor._info[0] for tensor in input_tensors_list]
+    next_depth = []
+    indegrees = {}
+
+    visited_layer_names = []
+    while not len(cur_depth) == 0:
+        node_by_depth.append(cur_depth)
+        for node in cur_depth:
+            if node.layer.name not in visited_layer_names:
+                all_layers.append(node.layer)
+                visited_layer_names.append(node.layer.name)
+            for out_node in node.out_nodes:
+                if out_node.node_name not in indegrees.keys():
+                    indegrees[out_node.node_name] = len(out_node.in_nodes)
+                indegrees[out_node.node_name] -= 1
+                if indegrees[out_node.node_name] == 0:
+                    next_depth.append(out_node)
+        cur_depth = next_depth
+        next_depth = []
+    return node_by_depth, all_layers
+
+
+def select_attrs(obj):
+    attrs_dict = obj.__dict__
+    attrs = {}
+    _select_key = ['kernel_size', 'stride', 'act', 'padding', 'data_format', 'concat_dim', 'dilation', 'bias']
+    for k in _select_key:
+        if k in attrs_dict:
+            if k == 'act':
+                attrs[k] = attrs_dict[k].__class__.__name__
+            else:
+                attrs[k] = attrs_dict[k]
+    return attrs
+
+
+class ModuleNode(object):
+    """
+    The class :class:`ModuleNode` class represents a conceptional node for a layer.
+
+    ModuleNode is used for building topology and it is actually a light weighted
+    wrapper over Layer.
+
+    Parameters
+    ----------
+    layer : tl.layers.Layer
+        A tl layer that wants to create a node.
+    node_index : int
+        Index of this node in layer._nodes.
+    in_nodes ：a list of ModuleNode
+        Father nodes to this node.
+    in_tensors : a list of tensors
+        Input tensors to this node.
+    out_tensors : a list of tensors
+        Output tensors to this node.
+    in_tensor_idxes : a list of int
+        Indexes of each input tensor in its corresponding node's out_tensors.
+
+    Methods
+    ---------
+    __init__()
+        Initializing the ModuleNode.
+    __call__()
+        (1) Forwarding through the layer. (2) Update its input/output tensors.
+    """
+
+    def __init__(self, layer, node_index, in_nodes, in_tensors, out_tensors, in_tensor_idxes, attr):
+        self.layer = layer
+        self.node_index = node_index
+        self.in_nodes = in_nodes
+        self.out_nodes = []
+        self.in_tensors = in_tensors
+        self.out_tensors = out_tensors
+        self.node_name = layer.name + "_node_{}".format(node_index)
+
+        self.in_tensors_idxes = in_tensor_idxes
+        self.attr = attr
+        self.visited = False
+
+    def __call__(self, inputs, **kwargs):
+        """(1) Forwarding through the layer. (2) Update its input/output tensors."""
+        outputs = self.layer(inputs, **kwargs)
+        self.in_tensors = tolist(inputs)
+        self.out_tensors = tolist(outputs)
+        return self.out_tensors
\ No newline at end of file
