@@ -1,21 +1,22 @@
 #! /usr/bin/python
 # -*- coding: utf-8 -*-
 
-from .common import str2act, str2init, random_normal
+from .common import check_parameter, str2act, str2init, random_normal, tolist, construct_graph, ModuleNode, select_attrs
 from .common import _save_weights, _load_weights, _save_standard_weights_dict, _load_standard_weights_dict
 from mindspore.nn import Cell
 import tensorlayerx as tlx
-from collections import OrderedDict
+import mindspore as ms
 from mindspore import log as logger
 import inspect
 from mindspore import context
 import numpy
 from mindspore.common.api import _pynative_executor
 from mindspore.common.parameter import Parameter
-
-__all__ = ['Module', 'Sequential', 'ModuleList']
+from collections import OrderedDict, abc as container_abcs
+__all__ = ['Module', 'Sequential', 'ModuleList', 'ModuleDict']
 
 _global_layer_name_dict = {}
+_global_layer_node = []
 
 
 class Module(Cell):
@@ -64,9 +65,10 @@ class Module(Cell):
 
         # Layer building state
         self._built = False
+
         # Layer nodes state
-        self._nodes = []
         self._nodes_fixed = False
+        self._build_graph = False
 
         # Layer weight state
         self._all_weights = []
@@ -91,10 +93,7 @@ class Module(Cell):
     def build(self, inputs_shape):
         raise Exception("The build(self, inputs_shape) method must be implemented by inherited class")
 
-    def _get_weights(
-        self, var_name, shape, init=random_normal(), trainable=True, transposed=False,
-        order=False
-    ):
+    def _get_weights(self, var_name, shape, init=random_normal(), trainable=True, transposed=False, order=False):
         """ Get trainable variables. """
         var_name = self.name + "/" + var_name
         # TODO 2D mindspore weights shape : [out_channel, in_channel, kernel_h, kernel_w]
@@ -106,7 +105,7 @@ class Module(Cell):
         if len(shape) == 3:
             shape = shape[::-1]
         if len(shape) == 4:
-            if not transposed and self.data_format == 'NHWC':
+            if not transposed and self.data_format in ['NHWC', 'channels_last']:
                 shape = (shape[3], shape[0], shape[1], shape[2])
             else:
                 shape = (shape[3], shape[2], shape[0], shape[1])
@@ -140,18 +139,24 @@ class Module(Cell):
             shape_mem = tlx.get_tensor_shape(tensors)
         return shape_mem
 
-    def __call__(self, *inputs, **kwargs):
+    def __call__(self, *args, **kwargs):
         if self.__class__.construct is Cell.construct:
             logger.warning(f"The '{self.__class__}' does not override the method 'construct', "
                            f"will call the super class(Cell) 'construct'.")
         if kwargs:
-            bound_args = inspect.signature(self.construct).bind(*inputs, **kwargs)
-            inputs = bound_args.args
-            kwargs = bound_args.kwargs
+            bound_arguments = inspect.signature(self.construct).bind(*args, **kwargs)
+            bound_arguments.apply_defaults()
+            args = bound_arguments.args
+            kwargs = bound_arguments.kwargs
 
         # Run in Graph mode.
-        if context.get_context("mode") == context.GRAPH_MODE:
-            raise NotImplemented("GRAPH MODE is not supported, please select PYNATIVE MODE.")
+        if context._get_mode() == context.GRAPH_MODE:
+            self._check_construct_args(*args, **kwargs)
+            if self.enable_hook:
+                raise ValueError("For 'Cell', it's not support hook function in graph mode, please use "
+                                 "context.set_context to set pynative mode.")
+            out = self.compile_and_run(*args)
+            return out
 
         # Run in PyNative mode.
         if _pynative_executor.is_top_cell():
@@ -159,20 +164,15 @@ class Module(Cell):
             # There many Casts in parameter_broadcast. Enable lazy_build and build faster.
             self._do_parameter_broadcast()
 
-        for item in inputs:
-            if isinstance(item, numpy.ndarray):
-                raise TypeError("The cell inputs should not be numpy arrays.")
+        for item in args:
+            if isinstance(item, ms.Tensor) and item.has_init:
+                item.init_data()
+            elif isinstance(item, numpy.ndarray):
+                raise TypeError("For 'Cell', inputs should not be numpy array.")
         if self.requires_grad is True:
             _pynative_executor.set_grad_flag(True)
-        _pynative_executor.new_graph(self, *inputs, **kwargs)
-        cast_inputs = list()
-        if hasattr(self, "_mindspore_flags"):
-            if self._mindspore_flags.get('fp16'):
-                cast_inputs = self._cast_mixed_precision_inputs(inputs, tlx.float16)
-            if self._mindspore_flags.get('fp32'):
-                cast_inputs = self._cast_mixed_precision_inputs(inputs, tlx.float32)
-        if not cast_inputs:
-            cast_inputs = inputs
+        _pynative_executor.new_graph(self, *args, **kwargs)
+        cast_inputs = self.auto_cast_inputs(args)
 
         with self.CellGuard():
             try:
@@ -182,29 +182,13 @@ class Module(Cell):
                 raise err
 
         if _pynative_executor.is_top_cell():
-            _pynative_executor.execute_all_task()
+            _pynative_executor.execute_lazy_task()
 
         if isinstance(output, Parameter):
             output = output.data
-        _pynative_executor.end_graph(self, output, *inputs, **kwargs)
+        _pynative_executor.end_graph(self, output, *args, **kwargs)
         return output
 
-    def _add_node(self, input_tensors, output_tensors):
-        """Add a LayerNode for this layer given input_tensors, output_tensors.
-
-        WARINING: This function should not be called from outside, it should only be called
-        in layer.__call__ when building static model.
-
-        Parameters
-        ----------
-        input_tensors : Tensor or a list of tensors
-            Input tensors to this layer.
-        output_tensors : Tensor or a list of tensors
-            Output tensors to this layer.
-
-        """
-        raise NotImplementedError
-
     def set_train(self):
         """
         Sets the cell to training mode.
@@ -281,6 +265,9 @@ class Module(Cell):
     def str_to_init(self, initializer):
         return str2init(initializer)
 
+    def check_param(self, param, dim='2d'):
+        return check_parameter(param, dim)
+
     def insert_child_to_layer(self, child_name, child):
         """
         Adds a child layer to the current layer.
@@ -310,6 +297,50 @@ class Module(Cell):
 
         self.forward(*inputs, **kwargs)
 
+    def build_graph(self, *inputs, **kwargs):
+        # Add nodes only when the composition is needed.
+        layers = self.cells_and_names(name_prefix='')
+        for layer_name, layer in layers:
+            if isinstance(layer, Module):
+                layer._build_graph = True
+        self.set_eval()
+
+        outputs = self.forward(*inputs, **kwargs)
+        self.inputs = inputs
+        self.outputs = outputs
+        self._node_by_depth, self._all_layers = construct_graph(self.inputs, self.outputs)
+        return self._node_by_depth, self._all_layers
+
+    def _add_node(self, input_tensors, output_tensors):
+        """Add a ModuleNode for this layer given input_tensors, output_tensors.
+
+        This function should not be called from outside, it should only be called
+        in __call__ when building static model.
+
+        Parameters
+        ----------
+        input_tensors : Tensor or a list of tensors
+            Input tensors to this layer.
+        output_tensors : Tensor or a list of tensors
+            Output tensors to this layer.
+
+        """
+        inputs_list = tolist(input_tensors)
+        outputs_list = tolist(output_tensors)
+        if self.__class__.__name__ in tlx.layers.inputs.__all__:
+            # for InputLayer, there should be no in_nodes
+            in_nodes = []
+            in_tensor_idxes = [0]
+        else:
+            in_nodes = [tensor._info[0] for tensor in inputs_list]
+            in_tensor_idxes = [tensor._info[1] for tensor in inputs_list]
+        node_index = len(_global_layer_node)
+
+        new_node = ModuleNode(self, node_index, in_nodes, inputs_list, outputs_list, in_tensor_idxes, select_attrs(self))
+        _global_layer_node.append(new_node)
+        for idx, tensor in enumerate(outputs_list):
+            tensor._info = (new_node, idx)
+
 
 class Sequential(Module):
     """
@@ -450,9 +481,10 @@ class ModuleList(Module):
     >>> layer_list.append(d3)
     """
 
-    def __init__(self, args):
-        Module.__init__(self)
-        self.extend(args)
+    def __init__(self, modules = None):
+        super(ModuleList, self).__init__()
+        if modules is not None:
+            self.extend(modules)
 
     def __getitem__(self, index):
         if isinstance(index, slice):
@@ -525,6 +557,91 @@ class ModuleList(Module):
         raise NotImplementedError
 
 
+class ModuleDict(Module):
+
+    def __init__(self, modules = None):
+        super(ModuleDict, self).__init__()
+        if modules is not None:
+            self.update(modules)
+
+    def __getitem__(self, key):
+
+        return self._cells[key]
+
+    def __setitem__(self, key, module):
+        if not isinstance(key, str):
+            raise TypeError("module name should be a string, but got {}".format(type(key)))
+        elif '.' in key:
+            raise KeyError("module name can't contain \".\", got: {}".format(key))
+        elif key == '':
+            raise KeyError("module name can't be empty string \"\"")
+        if _valid_module(module):
+            self._cells[key] = module
+
+    def __delitem__(self, key):
+
+        del self._cells[key]
+
+    def __len__(self):
+
+        return len(self._cells)
+
+    def __iter__(self):
+
+        return iter(self._cells)
+
+    def __contains__(self, key):
+
+        return key in self._cells
+
+    def clear(self):
+
+        self._cells.clear()
+
+    def pop(self, key):
+
+        temp = self[key]
+        del self[key]
+        return temp
+
+    def keys(self):
+
+        return self._cells.keys()
+
+    def items(self):
+
+        return self._cells.items()
+
+    def values(self):
+
+        return self._cells.values()
+
+    def update(self, modules):
+
+        if not isinstance(modules, container_abcs.Iterable):
+            raise TypeError(
+                "ModuleDict.update should be called with an "
+                "iterable of key/value pairs, but got " + type(modules).__name__
+            )
+        if isinstance(modules, (OrderedDict, ModuleDict, container_abcs.Mapping)):
+            for key, module in modules.items():
+                self[key] = module
+
+        else:
+            for j, m in enumerate(modules):
+                if not isinstance(m, container_abcs.Iterable):
+                    raise TypeError(
+                        "ModuleDict update sequence element "
+                        "#" + str(j) + " should be Iterable; is" + type(m).__name__
+                    )
+                if not len(m) == 2:
+                    raise ValueError(
+                        "ModuleDict update sequence element "
+                        "#" + str(j) + " has length " + str(len(m)) + "; 2 is required"
+                    )
+                self[m[0]] = m[1]
+
+
 def _valid_index(layer_num, index):
     if not isinstance(index, int):
         raise TypeError("Index {} is not int type")
