@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 import copy, six
-from .common import str2act, str2init
+from .common import check_parameter, str2act, str2init, tolist, construct_graph, ModuleNode
 from .common import _save_weights, _load_weights, _save_standard_weights_dict, _load_standard_weights_dict
 from paddle.fluid import framework
 from paddle.fluid.dygraph import Layer
@@ -11,10 +11,14 @@ from paddle.fluid.dygraph.base import program_desc_tracing_guard, param_guard
 from paddle.fluid.dygraph import parallel_helper
 import paddle as pd
 from collections import OrderedDict
+from collections import OrderedDict, abc as container_abcs
+import tensorlayerx as tlx
+from queue import Queue
 
 _global_layer_name_dict = {}
+_global_layer_node = []
 
-__all__ = ['Module', 'Sequential', 'ModuleList']
+__all__ = ['Module', 'Sequential', 'ModuleList', 'ModuleDict']
 
 
 class Module(Layer):
@@ -65,8 +69,8 @@ class Module(Layer):
         self._paddle_built = False
 
         # Layer nodes state
-        self._nodes = []
         self._nodes_fixed = False
+        self._build_graph = False
 
         # Layer weight state
         self._all_weights = None
@@ -284,6 +288,9 @@ class Module(Layer):
     def str_to_init(self, initializer):
         return str2init(initializer)
 
+    def check_param(self, param, dim='2d'):
+        return check_parameter(param, dim)
+
     def insert_child_to_layer(self, child_name, child):
         """
         Adds a child layer to the current layer.
@@ -305,6 +312,50 @@ class Module(Layer):
             raise TypeError("Child layer type is incorrect.")
         self._sub_layers[child_name] = child
 
+    def build_graph(self, *inputs, **kwargs):
+        # Add nodes only when the composition is needed.
+        # for layer in self.sublayers():
+        #     if isinstance(layer, Module):
+        #         layer._build_graph = True
+        #
+        # outputs = self.forward(*inputs, **kwargs)
+        # self.inputs = inputs
+        # self.outputs = outputs
+        # self._node_by_depth, self._all_layers = construct_graph(self.inputs, self.outputs)
+        # return self._node_by_depth, self._all_layers
+        raise NotImplementedError
+
+    def _add_node(self, input_tensors, output_tensors):
+        """Add a ModuleNode for this layer given input_tensors, output_tensors.
+
+        This function should not be called from outside, it should only be called
+        in __call__ when building static model.
+
+        Parameters
+        ----------
+        input_tensors : Tensor or a list of tensors
+            Input tensors to this layer.
+        output_tensors : Tensor or a list of tensors
+            Output tensors to this layer.
+
+        """
+        pass
+        # inputs_list = tolist(input_tensors)
+        # outputs_list = tolist(output_tensors)
+        # if self.__class__.__name__ in tlx.layers.inputs.__all__:
+        #     # for InputLayer, there should be no in_nodes
+        #     in_nodes = []
+        #     in_tensor_idxes = [0]
+        # else:
+        #     in_nodes = [tensor[0] for tensor in inputs_list]
+        #     in_tensor_idxes = [tensor[1] for tensor in inputs_list]
+        # node_index = len(_global_layer_node)
+        #
+        # new_node = ModuleNode(self, node_index, in_nodes, inputs_list, outputs_list, in_tensor_idxes)
+        # _global_layer_node.append(new_node)
+        # for idx, tensor in enumerate(outputs_list):
+        #     tensor._info = (new_node, idx)
+
 
 class Sequential(Module):
 
@@ -368,10 +419,10 @@ class Sequential(Module):
 
 class ModuleList(Module):
 
-    def __init__(self, args):
+    def __init__(self, modules = None):
         super(ModuleList, self).__init__()
-        if args is not None:
-            self.extend(args)
+        if modules is not None:
+            self.extend(modules)
 
     def __getitem__(self, index):
         if isinstance(index, slice):
@@ -451,6 +502,90 @@ class ModuleList(Module):
         raise NotImplementedError
 
 
+class ModuleDict(Module):
+
+    def __init__(self, modules = None):
+        super(ModuleDict, self).__init__()
+        if modules is not None:
+            self.update(modules)
+
+    def __getitem__(self, key):
+
+        return self._sub_layers[key]
+
+    def __setitem__(self, key, module):
+        if not isinstance(key, str):
+            raise TypeError("module name should be a string, but got {}".format(type(key)))
+        elif '.' in key:
+            raise KeyError("module name can't contain \".\", got: {}".format(key))
+        elif key == '':
+            raise KeyError("module name can't be empty string \"\"")
+        if _valid_module(module):
+            self._sub_layers[key] = module
+
+    def __delitem__(self, key):
+
+        del self._sub_layers[key]
+
+    def __len__(self):
+
+        return len(self._sub_layers)
+
+    def __iter__(self):
+
+        return iter(self._sub_layers)
+
+    def __contains__(self, key):
+
+        return key in self._sub_layers
+
+    def clear(self):
+
+        self._sub_layers.clear()
+
+    def pop(self, key):
+
+        temp = self[key]
+        del self[key]
+        return temp
+
+    def keys(self):
+
+        return self._sub_layers.keys()
+
+    def items(self):
+
+        return self._sub_layers.items()
+
+    def values(self):
+
+        return self._sub_layers.values()
+
+    def update(self, modules):
+
+        if not isinstance(modules, container_abcs.Iterable):
+            raise TypeError(
+                "ModuleDict.update should be called with an "
+                "iterable of key/value pairs, but got " + type(modules)
+            )
+        if isinstance(modules, (OrderedDict, ModuleDict, container_abcs.Mapping)):
+            for key, module in modules.items():
+                self[key] = module
+        else:
+            for j, m in enumerate(modules):
+                if not isinstance(m, container_abcs.Iterable):
+                    raise TypeError(
+                        "ModuleDict update sequence element "
+                        "#" + str(j) + " should be Iterable; is" + type(m)
+                    )
+                if not len(m) == 2:
+                    raise ValueError(
+                        "ModuleDict update sequence element "
+                        "#" + str(j) + " has length " + str(len(m)) + "; 2 is required"
+                    )
+                self[m[0]] = m[1]
+
+
 def _valid_index(layer_num, index):
     if not isinstance(index, int):
         raise TypeError("Index {} is not int type")
