@@ -1,17 +1,19 @@
 #! /usr/bin/python
 # -*- coding: utf-8 -*-
 
-from .common import str2act, str2init
+from .common import check_parameter, str2act, str2init, tolist, construct_graph, ModuleNode, select_attrs
 from .common import _save_weights, _load_weights, _save_standard_weights_dict, _load_standard_weights_dict
+from collections import OrderedDict, abc as container_abcs
 from collections import OrderedDict
 import time
 import tensorlayerx as tlx
 import tensorflow as tf
 from tensorlayerx.nn.layers.utils import (get_variable_with_initializer, random_normal)
 
-__all__ = ['Module', 'Sequential', 'ModuleList']
+__all__ = ['Module', 'Sequential', 'ModuleList', 'ModuleDict']
 
 _global_layer_name_dict = {}
+_global_layer_node = []
 
 
 class Module(object):
@@ -98,8 +100,8 @@ class Module(object):
         self._built = False
 
         # Layer nodes state
-        self._nodes = []
         self._nodes_fixed = False
+        self._build_graph = False
 
         # Layer weight state
         self._all_weights = None
@@ -322,23 +324,6 @@ class Module(object):
         except:
             pass
 
-    def _add_node(self, input_tensors, output_tensors):
-        """Add a LayerNode for this layer given input_tensors, output_tensors.
-
-        WARINING: This function should not be called from outside, it should only be called
-        in layer.__call__ when building static model.
-
-        Parameters
-        ----------
-        input_tensors : Tensor or a list of tensors
-            Input tensors to this layer.
-        output_tensors : Tensor or a list of tensors
-            Output tensors to this layer.
-
-        """
-
-        raise NotImplementedError
-
     @property
     def create_time(self):
         return self._create_time
@@ -597,6 +582,53 @@ class Module(object):
     def str_to_init(self, initializer):
         return str2init(initializer)
 
+    def check_param(self, param, dim='2d'):
+        return check_parameter(param, dim)
+
+    def build_graph(self, *inputs, **kwargs):
+        # Add nodes only when the composition is needed.
+        layers = self.layers_and_names(name_prefix='')
+        for layer_name, layer in layers:
+            if isinstance(layer, Module):
+                layer._build_graph = True
+        self.set_eval()
+
+        outputs = self.forward(*inputs, **kwargs)
+        self.inputs = inputs
+        self.outputs = outputs
+        self._node_by_depth, self._all_layers = construct_graph(self.inputs, self.outputs)
+        return self._node_by_depth, self._all_layers
+
+    def _add_node(self, input_tensors, output_tensors):
+        """Add a ModuleNode for this layer given input_tensors, output_tensors.
+
+        This function should not be called from outside, it should only be called
+        in __call__ when building static model.
+
+        Parameters
+        ----------
+        input_tensors : Tensor or a list of tensors
+            Input tensors to this layer.
+        output_tensors : Tensor or a list of tensors
+            Output tensors to this layer.
+
+        """
+        inputs_list = tolist(input_tensors)
+        outputs_list = tolist(output_tensors)
+        if self.__class__.__name__ in tlx.layers.inputs.__all__:
+            # for InputLayer, there should be no in_nodes
+            in_nodes = []
+            in_tensor_idxes = [0]
+        else:
+            in_nodes = [tensor._info[0] for tensor in inputs_list]
+            in_tensor_idxes = [tensor._info[1] for tensor in inputs_list]
+        node_index = len(_global_layer_node)
+
+        new_node = ModuleNode(self, node_index, in_nodes, inputs_list, outputs_list, in_tensor_idxes, select_attrs(self))
+        _global_layer_node.append(new_node)
+        for idx, tensor in enumerate(outputs_list):
+            tensor._info = (new_node, idx)
+
 
 class Sequential(Module):
     """
@@ -706,7 +738,7 @@ class ModuleList(Module):
     Methods
     ---------
     __init__()
-        Initializing the Layer.
+        Initializing the ModuleList.
     insert()
         Inserts a given layer before a given index in the list.
     extend()
@@ -731,9 +763,10 @@ class ModuleList(Module):
     >>> layer_list.append(d3)
     """
 
-    def __init__(self, args):
+    def __init__(self, modules = None):
         super(ModuleList, self).__init__()
-        self.extend(args)
+        if modules is not None:
+            self.extend(modules)
 
     def __getitem__(self, index):
         if isinstance(index, slice):
@@ -813,6 +846,138 @@ class ModuleList(Module):
         raise NotImplementedError
 
 
+class ModuleDict(Module):
+    """
+    Holds submodules in a dictionary.
+
+    ModuleDict can be used like a regular Python dictionary, support
+    '__getitem__', '__setitem__', '__delitem__', '__len__', '__iter__' and '__contains__',
+    but module it contains are properly registered, and will be visible by all Modules methods.
+
+    Parameters
+    ----------
+        args : dict
+            a mapping (dictionary) of (string: module)
+            or an iterable of key-value pairs of type (string, module)
+
+    Methods
+    ---------
+    __init__()
+        Initializing the ModuleDict.
+    clear()
+        Remove all items from the ModuleDict.
+    pop()
+        Remove key from the ModuleDict and return its module.
+    keys()
+        Return an iterable of the ModuleDict keys.
+    items()
+        Return an iterable of the ModuleDict key/value pairs.
+    values()
+        Return an iterable of the ModuleDict values.
+    update()
+        Update the ModuleDict with the key-value pairs from a
+        mapping or an iterable, overwriting existing keys.
+
+    Examples
+    ---------
+    >>> from tensorlayerx.nn import Module, ModuleDict, Linear
+    >>> import tensorlayerx as tlx
+    >>> class MyModule(Module):
+    >>>     def __init__(self):
+    >>>         super(MyModule, self).__init__()
+    >>>         self.dict = ModuleDict({
+    >>>                 'linear1':Linear(out_features=800, act=tlx.ReLU, in_features=784, name='linear1'),
+    >>>                 'linear2':Linear(out_features=800, act=tlx.ReLU, in_features=800, name='linear2')
+    >>>                 })
+    >>>     def forward(self, x, linear):
+    >>>         x = self.dict[linear](x)
+    >>>         return x
+
+    """
+
+    def __init__(self, modules = None):
+        super(ModuleDict, self).__init__()
+        if modules is not None:
+            self.update(modules)
+
+    def __getitem__(self, key):
+
+        return self._layers[key]
+
+    def __setitem__(self, key, module):
+        if not isinstance(key, str):
+            raise TypeError("module name should be a string, but got {}".format(type(key)))
+        elif '.' in key:
+            raise KeyError("module name can't contain \".\", got: {}".format(key))
+        elif key == '':
+            raise KeyError("module name can't be empty string \"\"")
+        if _valid_module(module):
+            self._layers[key] = module
+
+    def __delitem__(self, key):
+
+        del self._layers[key]
+
+    def __len__(self):
+
+        return len(self._layers)
+
+    def __iter__(self):
+
+        return iter(self._layers)
+
+    def __contains__(self, key):
+
+        return key in self._layers
+
+    def clear(self):
+
+        self._layers.clear()
+
+    def pop(self, key):
+
+        temp = self[key]
+        del self[key]
+        return temp
+
+    def keys(self):
+
+        return self._layers.keys()
+
+    def items(self):
+
+        return self._layers.items()
+
+    def values(self):
+
+        return self._layers.values()
+
+    def update(self, modules):
+
+        if not isinstance(modules, container_abcs.Iterable):
+            raise TypeError(
+                "ModuleDict.update should be called with an "
+                "iterable of key/value pairs, but got " + type(modules).__name__
+            )
+        if isinstance(modules, (OrderedDict, ModuleDict, container_abcs.Mapping)):
+            for key, module in modules.items():
+                self[key] = module
+
+        else:
+            for j, m in enumerate(modules):
+                if not isinstance(m, container_abcs.Iterable):
+                    raise TypeError(
+                        "ModuleDict update sequence element "
+                        "#" + str(j) + " should be Iterable; is" + type(m).__name__
+                    )
+                if not len(m) == 2:
+                    raise ValueError(
+                        "ModuleDict update sequence element "
+                        "#" + str(j) + " has length " + str(len(m)) + "; 2 is required"
+                    )
+                self[m[0]] = m[1]
+
+
 def _valid_index(layer_num, index):
     if not isinstance(index, int):
         raise TypeError("Index {} is not int type")
