@@ -2,17 +2,20 @@
 # -*- coding: utf-8 -*-
 
 from torch.nn import Module as T_Module
-from .common import str2act, str2init
+from .common import check_parameter, str2act, str2init, tolist, construct_graph, ModuleNode, select_attrs
 from .common import _save_weights, _load_weights, _save_standard_weights_dict, _load_standard_weights_dict
 from torch.nn.parameter import Parameter
 from collections import OrderedDict
 import torch
 import operator
 from itertools import islice
+from collections import OrderedDict, abc as container_abcs
+import tensorlayerx as tlx
 
 _global_layer_name_dict = {}
+_global_layer_node = []
 
-__all__ = ['Module', 'Sequential', 'ModuleList']
+__all__ = ['Module', 'Sequential', 'ModuleList', 'ModuleDict']
 
 
 class Module(T_Module):
@@ -61,8 +64,8 @@ class Module(T_Module):
         self._built = False
 
         # Layer nodes state
-        self._nodes = []
         self._nodes_fixed = False
+        self._build_graph = False
 
         # Layer weight state
         self._all_weights = None
@@ -170,6 +173,9 @@ class Module(T_Module):
     def str_to_init(self, initializer):
         return str2init(initializer)
 
+    def check_param(self, param, dim='2d'):
+        return check_parameter(param, dim)
+
     def init_build(self, *inputs, **kwargs):
         """
         (1) This method must be called when the Layer has no input in_channels.
@@ -178,6 +184,50 @@ class Module(T_Module):
 
         self.forward(*inputs, **kwargs)
 
+    def build_graph(self, *inputs, **kwargs):
+        # Add nodes only when the composition is needed.
+        for name, layer in self.named_modules():
+            if isinstance(layer, Module):
+                layer._build_graph = True
+        self.set_eval()
+
+        outputs = self.forward(*inputs, **kwargs)
+        self.inputs = inputs
+        self.outputs = outputs
+        self._node_by_depth, self._all_layers = construct_graph(self.inputs, self.outputs)
+        return self._node_by_depth, self._all_layers
+
+    def _add_node(self, input_tensors, output_tensors):
+        """Add a ModuleNode for this layer given input_tensors, output_tensors.
+
+        This function should not be called from outside, it should only be called
+        in __call__ when building static model.
+
+        Parameters
+        ----------
+        input_tensors : Tensor or a list of tensors
+            Input tensors to this layer.
+        output_tensors : Tensor or a list of tensors
+            Output tensors to this layer.
+
+        """
+
+        inputs_list = tolist(input_tensors)
+        outputs_list = tolist(output_tensors)
+        if self.__class__.__name__ in tlx.layers.inputs.__all__:
+            # for InputLayer, there should be no in_nodes
+            in_nodes = []
+            in_tensor_idxes = [0]
+        else:
+            in_nodes = [tensor._info[0] for tensor in inputs_list]
+            in_tensor_idxes = [tensor._info[1] for tensor in inputs_list]
+        node_index = len(_global_layer_node)
+
+        new_node = ModuleNode(self, node_index, in_nodes, inputs_list, outputs_list, in_tensor_idxes, select_attrs(self))
+        _global_layer_node.append(new_node)
+        for idx, tensor in enumerate(outputs_list):
+            tensor._info = (new_node, idx)
+
 
 class Sequential(Module):
     """
@@ -329,9 +379,10 @@ class ModuleList(Module):
     >>> layer_list.append(d3)
     """
 
-    def __init__(self, args):
+    def __init__(self, modules = None):
         super(ModuleList, self).__init__()
-        self.extend(args)
+        if modules is not None:
+            self.extend(modules)
 
     def __getitem__(self, index):
         if isinstance(index, slice):
@@ -411,6 +462,84 @@ class ModuleList(Module):
         raise NotImplementedError
 
 
+class ModuleDict(Module):
+
+    def __init__(self, modules = None):
+        super(ModuleDict, self).__init__()
+        if modules is not None:
+            self.update(modules)
+
+    def __getitem__(self, key):
+
+        return self._modules[key]
+
+    def __setitem__(self, key, module):
+
+        self.add_module(key, module)
+
+    def __delitem__(self, key):
+
+        del self._modules[key]
+
+    def __len__(self):
+
+        return len(self._modules)
+
+    def __iter__(self):
+
+        return iter(self._modules)
+
+    def __contains__(self, key):
+
+        return key in self._modules
+
+    def clear(self):
+
+        self._modules.clear()
+
+    def pop(self, key):
+
+        v = self[key]
+        del self[key]
+        return v
+
+    def keys(self):
+
+        return self._modules.keys()
+
+    def items(self):
+
+        return self._modules.items()
+
+    def values(self):
+
+        return self._modules.values()
+
+    def update(self, modules):
+        if not isinstance(modules, container_abcs.Iterable):
+            raise TypeError(
+                "ModuleDict.update should be called with an "
+                "iterable of key/value pairs, but got " + type(modules).__name__
+            )
+
+        if isinstance(modules, (OrderedDict, ModuleDict, container_abcs.Mapping)):
+            for key, module in modules.items():
+                self[key] = module
+        else:
+            for j, m in enumerate(modules):
+                if not isinstance(m, container_abcs.Iterable):
+                    raise TypeError(
+                        "ModuleDict update sequence element "
+                        "#" + str(j) + " should be Iterable; is" + type(m).__name__
+                    )
+                if not len(m) == 2:
+                    raise ValueError(
+                        "ModuleDict update sequence element "
+                        "#" + str(j) + " has length " + str(len(m)) + "; 2 is required"
+                    )
+                self[m[0]] = m[1]
+
+
 def _valid_index(layer_num, index):
     if not isinstance(index, int):
         raise TypeError("Index {} is not int type")
