@@ -30,7 +30,7 @@ class Conv1d(Module):
         Specifying the dilation rate to use for dilated convolution.
     act : activation function
         The function that is applied to the layer activations
-    padding : str
+    padding : str or int
         The padding algorithm type: "SAME" or "VALID".
     data_format : str
         "channel_last" (NWC, default) or "channels_first" (NCW).
@@ -109,11 +109,9 @@ class Conv1d(Module):
 
     def build(self, inputs_shape):
         if self.data_format == 'channels_last':
-            self.data_format = 'NWC'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[-1]
         elif self.data_format == 'channels_first':
-            self.data_format = 'NCW'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[1]
         else:
@@ -152,6 +150,9 @@ class Conv1d(Module):
         if self.act_init_flag:
             outputs = self.act(outputs)
 
+        if not self._nodes_fixed and self._build_graph:
+            self._add_node(inputs, outputs)
+            self._nodes_fixed = True
         return outputs
 
 
@@ -162,17 +163,18 @@ class Conv2d(Module):
     ----------
     out_channels  : int
         Number of channels produced by the convolution
-    kernel_size : tuple of int
+    kernel_size : tuple or int
         The kernel size (height, width).
-    stride : tuple of int
+    stride : tuple or int
         The sliding window stride of corresponding input dimensions.
         It must be in the same order as the ``shape`` parameter.
-    dilation : tuple of int
+    dilation : tuple or int
         Specifying the dilation rate to use for dilated convolution.
     act : activation function
         The activation function of this layer.
-    padding : str
+    padding : int, tuple or str
         The padding algorithm type: "SAME" or "VALID".
+        If padding is int or tuple, padding added to all four sides of the input. Default: ‘SAME’
     data_format : str
         "channels_last" (NHWC, default) or "channels_first" (NCHW).
     W_init : initializer or str
@@ -212,11 +214,11 @@ class Conv2d(Module):
     ):
         super(Conv2d, self).__init__(name, act=act)
         self.out_channels = out_channels
-        self.kernel_size = kernel_size
-        self._strides = self.stride = stride
+        self.kernel_size = self.check_param(kernel_size)
+        self._strides = self.stride = self.check_param(stride)
         self.padding = padding
         self.data_format = data_format
-        self._dilation_rate = self.dilation = dilation
+        self._dilation_rate = self.dilation = self.check_param(dilation)
         self.W_init = self.str_to_init(W_init)
         self.b_init = self.str_to_init(b_init)
         self.in_channels = in_channels
@@ -250,13 +252,11 @@ class Conv2d(Module):
 
     def build(self, inputs_shape):
         if self.data_format == 'channels_last':
-            self.data_format = 'NHWC'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[-1]
             self._strides = [1, self._strides[0], self._strides[1], 1]
             self._dilation_rate = [1, self._dilation_rate[0], self._dilation_rate[1], 1]
         elif self.data_format == 'channels_first':
-            self.data_format = 'NCHW'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[1]
             self._strides = [1, 1, self._strides[0], self._strides[1]]
@@ -295,6 +295,10 @@ class Conv2d(Module):
             outputs = self.bias_add(outputs, self.b)
         if self.act_init_flag:
             outputs = self.act(outputs)
+
+        if not self._nodes_fixed and self._build_graph:
+            self._add_node(inputs, outputs)
+            self._nodes_fixed = True
         return outputs
 
 
@@ -305,16 +309,16 @@ class Conv3d(Module):
     ----------
     out_channels  : int
         Number of channels produced by the convolution
-    kernel_size : tuple of int
+    kernel_size : tuple or int
         The kernel size (depth, height, width).
-    stride : tuple of int
+    stride : tuple or int
         The sliding window stride of corresponding input dimensions.
         It must be in the same order as the ``shape`` parameter.
-    dilation : tuple of int
+    dilation : tuple or int
         Specifying the dilation rate to use for dilated convolution.
     act : activation function
         The activation function of this layer.
-    padding : str
+    padding : int, tuple or str
         The padding algorithm type: "SAME" or "VALID".
     data_format : str
         "channels_last" (NDHWC, default) or "channels_first" (NCDHW).
@@ -355,11 +359,11 @@ class Conv3d(Module):
     ):
         super().__init__(name, act=act)
         self.out_channels  = out_channels
-        self.kernel_size = kernel_size
-        self._strides = self.stride = stride
+        self.kernel_size = self.check_param(kernel_size, dim='3d')
+        self._strides = self.stride = self.check_param(stride, dim='3d')
         self.padding = padding
         self.data_format = data_format
-        self._dilation_rate = self.dilation = dilation
+        self._dilation_rate = self.dilation = self.check_param(dilation, dim='3d')
         self.W_init = self.str_to_init(W_init)
         self.b_init = self.str_to_init(b_init)
         self.in_channels = in_channels
@@ -393,13 +397,11 @@ class Conv3d(Module):
 
     def build(self, inputs_shape):
         if self.data_format == 'channels_last':
-            self.data_format = 'NDHWC'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[-1]
             self._strides = [1, self._strides[0], self._strides[1], self._strides[2], 1]
             self._dilation_rate = [1, self.dilation[0], self.dilation[1], self.dilation[2], 1]
         elif self.data_format == 'channels_first':
-            self.data_format = 'NCDHW'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[1]
             self._strides = [1, 1, self._strides[0], self._strides[1], self._strides[2]]
@@ -440,6 +442,10 @@ class Conv3d(Module):
             outputs = self.bias_add(outputs, self.b)
         if self.act_init_flag:
             outputs = self.act(outputs)
+
+        if not self._nodes_fixed and self._build_graph:
+            self._add_node(inputs, outputs)
+            self._nodes_fixed = True
         return outputs
 
 
@@ -537,11 +543,9 @@ class ConvTranspose1d(Module):
 
     def build(self, inputs_shape):
         if self.data_format == 'channels_last':
-            self.data_format = 'NWC'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[-1]
         elif self.data_format == 'channels_first':
-            self.data_format = 'NCW'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[1]
         else:
@@ -584,6 +588,10 @@ class ConvTranspose1d(Module):
             outputs = self.bias_add(outputs, self.b)
         if self.act_init_flag:
             outputs = self.act(outputs)
+
+        if not self._nodes_fixed and self._build_graph:
+            self._add_node(inputs, outputs)
+            self._nodes_fixed = True
         return outputs
 
 
@@ -594,12 +602,12 @@ class ConvTranspose2d(Module):
     ----------
     out_channels : int
         Number of channels produced by the convolution
-    kernel_size : tuple of int
+    kernel_size : tuple or int
         The kernel size (height, width).
-    stride : tuple of int
+    stride : tuple or int
         The sliding window stride of corresponding input dimensions.
         It must be in the same order as the ``shape`` parameter.
-    dilation : tuple of int
+    dilation : tuple or int
         Specifying the dilation rate to use for dilated convolution.
     act : activation function
         The activation function of this layer.
@@ -644,11 +652,11 @@ class ConvTranspose2d(Module):
     ):
         super(ConvTranspose2d, self).__init__(name, act=act)
         self.out_channels = out_channels
-        self.kernel_size = kernel_size
-        self.stride = stride
+        self.kernel_size = self.check_param(kernel_size)
+        self.stride = self.check_param(stride)
         self.padding = padding
         self.data_format = data_format
-        self.dilation = dilation
+        self.dilation = self.check_param(dilation)
         self.W_init = self.str_to_init(W_init)
         self.b_init = self.str_to_init(b_init)
         self.in_channels = in_channels
@@ -682,11 +690,9 @@ class ConvTranspose2d(Module):
 
     def build(self, inputs_shape):
         if self.data_format == 'channels_last':
-            self.data_format = 'NHWC'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[-1]
         elif self.data_format == 'channels_first':
-            self.data_format = 'NCHW'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[1]
         else:
@@ -723,6 +729,10 @@ class ConvTranspose2d(Module):
             outputs = self.bias_add(outputs, self.b)
         if self.act_init_flag:
             outputs = self.act(outputs)
+
+        if not self._nodes_fixed and self._build_graph:
+            self._add_node(inputs, outputs)
+            self._nodes_fixed = True
         return outputs
 
 
@@ -733,12 +743,12 @@ class ConvTranspose3d(Module):
     ----------
     out_channels : int
         Number of channels produced by the convolution
-    kernel_size : tuple of int
+    kernel_size : tuple or int
         The kernel size (depth, height, width).
-    stride : tuple of int
+    stride : tuple or int
         The sliding window stride of corresponding input dimensions.
         It must be in the same order as the ``shape`` parameter.
-    dilation : tuple of int
+    dilation : tuple or int
         Specifying the dilation rate to use for dilated convolution.
     act : activation function
         The activation function of this layer.
@@ -783,11 +793,11 @@ class ConvTranspose3d(Module):
     ):
         super(ConvTranspose3d, self).__init__(name, act=act)
         self.out_channels = out_channels
-        self.kernel_size = kernel_size
-        self.stride = stride
+        self.kernel_size = self.check_param(kernel_size, dim='3d')
+        self.stride = self.check_param(stride, dim='3d')
         self.padding = padding
         self.data_format = data_format
-        self.dilation = dilation
+        self.dilation = self.check_param(dilation, dim='3d')
         self.W_init = self.str_to_init(W_init)
         self.b_init = self.str_to_init(b_init)
         self.in_channels = in_channels
@@ -821,11 +831,9 @@ class ConvTranspose3d(Module):
 
     def build(self, inputs_shape):
         if self.data_format == 'channels_last':
-            self.data_format = 'NDHWC'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[-1]
         elif self.data_format == 'channels_first':
-            self.data_format = 'NCDHW'
             if self.in_channels is None:
                 self.in_channels = inputs_shape[1]
         else:
@@ -865,4 +873,8 @@ class ConvTranspose3d(Module):
             outputs = self.bias_add(outputs, self.b)
         if self.act_init_flag:
             outputs = self.act(outputs)
+
+        if not self._nodes_fixed and self._build_graph:
+            self._add_node(inputs, outputs)
+            self._nodes_fixed = True
         return outputs
