@@ -43,4 +43,8 @@ class Sign(Module):
 
     def forward(self, inputs):
         outputs = quantize(inputs)
+
+        if not self._nodes_fixed and self._build_graph:
+            self._add_node(inputs, outputs)
+            self._nodes_fixed = True
         return outputs
