@@ -4,7 +4,7 @@
 
 MAJOR = 0
 MINOR = 5
-PATCH = 1
+PATCH = 3
 PRE_RELEASE = ''
 # Use the following formatting: (major, minor, patch, prerelease)
 VERSION = (MAJOR, MINOR, PATCH, PRE_RELEASE)
@@ -18,7 +18,7 @@ __contact_emails__ = 'tensorlayerx@gmail.com'
 __homepage__ = 'https://tensorlayerx.readthedocs.io/en/latest/'
 __repository_url__ = 'https://github.com/tensorlayer/TensorLayerX'
 __download_url__ = 'https://github.com/tensorlayer/TensorLayerX'
-__description__ = 'High Level Tensorflow Deep Learning Library for Researcher and Engineer.'
+__description__ = 'High Level Deep Learning Library for Researcher and Engineer.'
 __license__ = 'apache'
 __keywords__ = 'deep learning, machine learning, computer vision, nlp, '
-__keywords__ += 'supervised learning, unsupervised learning, reinforcement learning, tensorflow'
+__keywords__ += 'supervised learning, unsupervised learning, reinforcement learning'
